Package: BaalChIP
Title: BaalChIP: Bayesian analysis of allele-specific transcription
        factor binding in cancer genomes
Description: The package offers functions to process multiple ChIP-seq
        BAM files and detect allele-specific events. Computes allele
        counts at individual variants (SNPs/SNVs), implements extensive
        QC steps to remove problematic variants, and utilizes a
        bayesian framework to identify statistically significant
        allele- specific events. BaalChIP is able to account for copy
        number differences between the two alleles, a known
        phenotypical feature of cancer samples.
Version: 1.36.0
Author: Ines de Santiago, Wei Liu, Ke Yuan, Martin O'Reilly, Chandra SR Chilamakuri, Bruce Ponder, Kerstin
        Meyer, Florian Markowetz
Maintainer: Ines de Santiago <inesdesantiago@gmail.com>
Depends: R (>= 3.3.1), GenomicRanges, IRanges, Rsamtools,
Imports: GenomicAlignments, GenomeInfoDb, doParallel, parallel, doBy,
        reshape2, scales, coda, foreach, ggplot2, methods, utils,
        graphics, stats
Suggests: RUnit, BiocGenerics, knitr, rmarkdown, BiocStyle
VignetteBuilder: knitr
License: Artistic-2.0
LazyData: true
RoxygenNote: 7.1.1
biocViews: Software, ChIPSeq, Bayesian, Sequencing
git_url: https://git.bioconductor.org/packages/BaalChIP
git_branch: RELEASE_3_22
git_last_commit: 4f4fba9
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 02:37:09 UTC; biocbuild
